#!/bin/bash

#
# QUICK START EXAMPLES:
#
# Using with no arguments gives USAGE INFO
# 
# ./newinstall.sh install all
#
# ./newinstall.sh check links
#
#

WORKDIR=`echo $0 |sed -e "s/\(.*\)\/\(.*\)$/\1/"`
source ${WORKDIR}/installLib.sh

set -x

#source installLib.sh

alias verify_link=isdir
alias verify_link=islink
alias verify_file=isfile
alias cp="ambiguous cp"
alias mv="ambiguous mv"
alias mkdir="ambiguous mkdir"
alias rm="ambigous rm"
alias ln="ambiguous ln"


#NOTE: This script is designed to be run from /tmp/ip_update/ipitomy/scripts/

ROOTDIR=/usr/local/ipitomy
TMPPKGDIR=/tmp/ip_update/ipitomy



########################################
# Should mimic behavior of original install.sh
#

function install_packages()
{


	if [ ! -d /usr/local/ipitomy/ssl ] 
	then
		mkdir -p /usr/local/ipitomy/ssl
		cp -rf $TMPPKGDIR/ssl/* /usr/local/ipitomy/ssl
	fi
	cd $TMPPKGDIR/RPMS

	
	# ensure all asterisk and zaptel processes are stopped
	kill_asterisk_and_zaptel


	
	# this will eventually be the yum update call
	update_rpms
	
	if [ -f /tmp/ipitomy_yum-current.i386.rpm ]
	then 
		rm -f /tmp/ipitomy_yum-current.i386.rpm
	fi
	curl -o /tmp/ipitomy_yum-current.i386.rpm https://ipitomy.biz/ipitomy_yum/os/CentOS/5.1/x86/ipitomy_yum-current.i386.rpm
	if [ $? -gt 0 ] 
	then
		TMPRET=$?
		echo "Error retrieving update file."	
		exit $TMPRET
	else
		rpm -Uvh /tmp/ipitomy_yum-current.i386.rpm
		rpm -qa | grep ipitomy_yum
		if [ $? -gt 0 ]
		then
			echo "Failed to install ipitomy repository RPM exiting"
			exit $?
		fi
		
	fi
	

	if [ $? == 0 ]
	then
		yum -y update 
		if [ ! $? == 0 ]
		then
			echo "WARNING: yum update returned non-zero"
		fi
		yum -y install ipitomy_pbx
		if [ ! $? == 0 ]
		then
			echo "WARNING: yum install ipitomy_pbx returned non-zero"
		fi
	fi
	
}


# this function will eventually use the yum repo - but just fudge it for now
function update_rpms()
{
	
	# install the kmod it has issues with localinstall and localupdate
	# kmod - if you install with rpm -Uvh this works if you update with 
    #        localupdate it continuously tries to update

	yum --disablerepo=base,updates,extras,addons,centosplus -y -c $TMPPKGDIR/RPMS/yum.conf localupdate *.rpm 
	yum --disablerepo=base,updates,extras,addons,centosplus -y -c $TMPPKGDIR/RPMS/yum.conf localinstall *.rpm
}

function kill_asterisk_and_zaptel()
{
	service asterisk stop
	killall -9 asterisk
	killall -9 safe_asterisk
	killall -9 asterisk
	service zaptel stop
	MODS="tor2 wct4xxp wcte11xp wcfxo wcfxs wctdm24xxp wcusb wctfxsusb torisa xpp_usb xpd_fxs xpd_fxo xpp ztdummy zaptel"
	for CMOD in $MODS
	do
		rmmod $CMOD	
	done
	
}



function update_all
{
	service httpd stop

	# move all yum repository config out of default dir
	mkdir -p /root/yum.repos.d-backup
	mv -f /etc/yum.repos.d/*.repo /root/yum.repos.d-backup

    set +x
    install_packages
    set -x

    cd ${WORKDIR}

	# backup previous yum configuration
	

	

    # removed this line from updates.  We will have to have a better way to fix this if it's modified
   	#cp_rf "/usr/local/ipitomy/etc/hosts.*" /etc/ #CHANGE TO BACKUP hosts file if it is changed above

}

