#!/bin/sh

# assume that there are only two (2) sata drives: /dev/sda & /dev/sdb
# assume that the new drive is not the one marked as 'active sync'


i=0 	# counter for while-loop represents the partition number of the RAID
ARRAY_DAMAGED=0		# set to 1 if the RAID array contains at least one damaged/removed drive
while [ $i -le 2 ]
do
	# we don't know what the new drive will look like in the mdadm --details
	# but we do know that a good drive will show 'active sync'
	GOOD_DRIVE=(`/sbin/mdadm --detail /dev/md$i | awk '/active sync/ { if ($7 ~ /sda/) print "/dev/sda"; else if ($7 ~ /sdb/) print "/dev/sdb"; }'`)

	# however, if both drives are good, the $GOOD_DRIVE will contain both drives
	# if there is a bad drive, then $GOOD_DRIVE will only have one drive
	if [ ${GOOD_DRIVE[1]}'txt' = "txt" ]
	then
		# leave if we find a RAID drive that has only one drive in it
		ARRAY_DAMAGED=1;
		break;
	fi
	i=$(( $i + 1 ));
	ARRAY_DAMAGED=0;	# can't hurt to keep it 0
done

# now, rebuild the array if a bad drive has been detected
if [ $ARRAY_DAMAGED -eq 1 ]
then
	BAD_DRIVE="unknown"
	if [ ${GOOD_DRIVE[0]} = "/dev/sda" ]
	then
		BAD_DRIVE="/dev/sdb";
	elif [ ${GOOD_DRIVE[0]} == "/dev/sdb" ]
	then
		BAD_DRIVE="/dev/sda";
	fi

	# guarantee that the new drive isn't being used
	/sbin/mdadm /dev/md0 --fail $BAD_DRIVE'1' --remove $BAD_DRIVE'1'
	/sbin/mdadm /dev/md1 --fail $BAD_DRIVE'2' --remove $BAD_DRIVE'2'
	/sbin/mdadm /dev/md2 --fail $BAD_DRIVE'3' --remove $BAD_DRIVE'3'

	# reinitialize partition tables
	sfdisk -d $GOOD_DRIVE | sfdisk $BAD_DRIVE

	# start rebuilding the array
	/sbin/mdadm /dev/md0 -a /dev/sda1
	/sbin/mdadm /dev/md0 -a /dev/sdb1
	/sbin/mdadm /dev/md1 -a /dev/sda2
	/sbin/mdadm /dev/md1 -a /dev/sdb2
	/sbin/mdadm /dev/md2 -a /dev/sda3
	/sbin/mdadm /dev/md2 -a /dev/sdb3
fi
