#!/bin/sh

# This script restarts Asterisk and Zaptel. Also registers all connected Astribanks in the proper order.
#
# You cannot test changes here manually. To be sure it works, you need to test via web page -Henry
#

LOG="/tmp/xorcom.messages"
TAG="/tmp/xorcom.done"
rm $LOG
touch $LOG

rm -f $TAG
 
function log
{
    tod=`date`
    chmod 666 $LOG
    echo "$tod xorcom_restart $*" >> $LOG
}


# ------ Identify order of Astribanks before we stop anything -- important
todo=`sudo /usr/sbin/zt_registration |grep "/XPD"|awk '{ print $1; }'|tr -d ":"`
countinput=`echo $todo|tr " " "\n"`
count=`echo $countinput|wc -l`

if [ -z "$countinput" ]; then
    count=0
fi

log "Identified $count Astribank Devices"

# -------------------------------

log "Stopping PBX service and Zaptel..."
sudo /sbin/service asterisk stop
sudo /sbin/service zaptel stop
sudo /sbin/rmmod xpp
sudo /sbin/rmmod zttranscode
sudo /sbin/service zaptel start

# --------- this is more reliable than zt_registration on !!  -Henry


sofar=0
for t in $todo; do
let sofar=sofar+1

bus=`echo $t| awk -F "/" '{print $1; }'`
module=`echo $t| awk -F "/" '{print $2; }'`
usbaddr=`sudo /usr/sbin/zt_registration|grep $bus|grep usb-|awk '{ print $3; }'`;

log "Starting $module [$usbaddr]"
echo 1 > /proc/xpp/$t/zt_registration
bus=`echo $t | tr "/" " " | awk '{ print $1; }'`
cat /proc/xpp/$bus/waitfor_xpds > /dev/null
done
# --------------------------------



log "Loading Hardware & Restarting PBX Services"
sudo /sbin/service zaptel start
sudo /sbin/service asterisk start


# ztcfg -v is NOT reliable from Apache and we wouldn't want to reload config anyhow.
# If asterisk is dead report it!

afterstart=`service asterisk status`
echo "$afterstart" | grep -v "running"
alive=$?

if [ "$alive" -ne "1" ]; then
    log "<b>PBX Service Restart Failed!</b>"
    log "<b>Make Sure Configuration Matches Hardware!</b>"
    log "<b>Did you forget to Apply Changes?</b>";
touch $TAG
exit
fi


log "Done"
touch $TAG
exit 0
